import fs from 'fs'
import path from 'path'
import archiver from 'archiver'
import * as tar from 'tar'

// read the package json
const packageJsonPath = path.resolve('./package.json')
// parse the content
const packageJson = JSON.parse(fs.readFileSync(packageJsonPath, 'utf-8'))
// read the project name
const projectName = packageJson.name.toUpperCase() || 'project'
// read the project version
const version = packageJson.version || '0.0.1'
// define the source dir
const sourceDir = './dist'
// define the target dir
const outputDir = './release'
// create the base output filename
const baseFileName = `${projectName}-${version}`
// create the output filename for the zip file
const zipOutput = path.join(outputDir, `${baseFileName}.zip`)
// create the output filename for the tar zip file
const tarOutput = path.join(outputDir, `${baseFileName}.tar.gz`)

// function to read the content of the source dir and put them in the zip file
async function createZip() {
	return new Promise((resolve, reject) => {
		// create an output file to stream the zip content into
		const output = fs.createWriteStream(zipOutput)
		// configure the zip file
		const archive = archiver('zip', {
			zlib: { level: 9 },
		})
		// define an async function to print a success message
		output.on('close', () => {
			console.log(`[√] ZIP created: ${zipOutput} (${archive.pointer()} bytes)`)
			resolve()
		})
		// define an async function to print an error message
		archive.on('error', (err) => reject(err))
		// pipe the output stream
		archive.pipe(output)
		// read the content of the source dir
		archive.directory(sourceDir, false)
		// close the zip stream and finish the file
		archive.finalize()
	})
}

// function to read the content of the source dir and put them in the tar.gz file 
async function createTarGz() {
	return tar.c(
		{
			gzip: true,
			file: tarOutput,
			cwd: path.dirname(sourceDir),
		},
		[path.basename(sourceDir)]
	).then(() => {
		console.log(`[√] tar.gz created: ${tarOutput}`)
	})
}

// Ausführung
async function run() {
	try {
		await fs.promises.mkdir(outputDir, { recursive: true })
		console.log(`[!] Using project: "${projectName}", version: "${version}"`)
		await createZip()
		await createTarGz()
	} catch (err) {
		console.error('[X] Error during compression:', err)
	}
}

run()
